"""
Tests for traits defined in mayavi.core.traits
"""
# Author: Prabhu Ramachandran <prabhu@aero.iitb.ac.in>
# Copyright (c) 2008, Prabhu Ramachandran Enthought, Inc.
# License: BSD Style.

import unittest
import numpy
from traits.api import (HasTraits, Either, Array, Any,
                TraitError, Float, Int)
from mayavi.core.trait_defs import (ArrayNumberOrNone, ArrayOrNone,
    ShadowProperty)


class DataNotSmart(HasTraits):
    x = ShadowProperty(ArrayOrNone, smart_notify=False)
    # Test attribute.
    _test = Any
    def _x_changed(self, value):
        self._test = value.copy()

class DataSmart(HasTraits):
    x = ShadowProperty(ArrayOrNone, smart_notify=True)
    # Test attribute.
    _test = Any
    def _x_changed(self, value):
        self._test = value.copy()

class Simple(HasTraits):
    x = ShadowProperty(Float)
    # Test attribute.
    _test = Int(0)
    def _x_changed(self, value):
        self._test += 1

class HasArrays(HasTraits):
    x = ArrayOrNone
    y = ArrayNumberOrNone

    # Test attribute.
    _test_x = Int(0)
    _test_y = Int(0)

    def _x_changed(self, value):
        self._test_x += 1

    def _y_changed(self, value):
        self._test_y += 1


class TestShadowProperty(unittest.TestCase):
    def test_simple_case(self):
        "Test a simple trait wrapped as a shadow property."
        s = Simple(x=10.0)
        self.assertEqual(s._test, 1)
        self.assertEqual(s.x, 10.0)
        s.x = 10.0
        self.assertEqual(s._test, 1)
        self.assertEqual(s.x, 10.0)
        s.x = 20.0
        self.assertEqual(s._test, 2)
        self.assertEqual(s.x, 20.0)
        self.assertRaises(TraitError, s.__setattr__, 'x', 'hey')

    def test_shadow_property_smart(self):
        "Test if the shadow property trait type works correctly."
        x = numpy.linspace(0, 1)
        d = DataSmart(x=x)
        self.assertEqual(numpy.all(d.x == x), True)
        self.assertEqual(numpy.all(d._x == x), True)
        self.assertEqual(numpy.all(d._test == x), True)
        old = x.copy()
        x *= 2
        d.x = x
        self.assertEqual(numpy.all(d.x == x), True)
        self.assertEqual(numpy.all(d._x == x), True)
        # Notifier shouldn't be called.
        self.assertEqual(numpy.all(d._test == old), True)

    def test_shadow_property_not_smart(self):
        "Test if the shadow property trait type works correctly."
        x = numpy.linspace(0, 1)
        d = DataNotSmart(x=x)
        self.assertEqual(numpy.all(d.x == x), True)
        self.assertEqual(numpy.all(d._x == x), True)
        self.assertEqual(numpy.all(d._test == x), True)
        x *= 2
        d.x = x
        self.assertEqual(numpy.all(d.x == x), True)
        self.assertEqual(numpy.all(d._x == x), True)
        self.assertEqual(numpy.all(d._test == x), True)

    def test_type_checking(self):
        "Test if the validation works correctly."
        x = numpy.linspace(0, 1)
        d = DataNotSmart(x=x)
        self.assertRaises(TraitError, d.__setattr__, 'x', 'hey')

    def test_set_trait_change_notify(self):
        "If trait_change_notify is honored."
        s = Simple()
        trait_names = s.trait_names()
        s.x = 10.0
        self.assertEqual(s._test, 1)
        self.assertEqual(s.x, 10.0)
        s.trait_set(x=20.0, trait_change_notify=False)
        self.assertEqual(s._test, 1)
        self.assertEqual(s.x, 20.0)
        # Assert that no new traits were added and no new notifiers were
        # added.
        self.assertEqual(s.trait_names(), trait_names)
        self.assertEqual(s._notifiers(False), None)


class TestArrayOrNone(unittest.TestCase):

    def test_default(self):
        a = HasArrays()
        self.assertIsNone(a.x)
        self.assertIsNone(a.y)

    def test_no_compare(self):
        a = HasArrays()
        a.x = numpy.arange(10)
        self.assertEqual(a._test_x, 1)
        a.x = numpy.arange(10)
        self.assertEqual(a._test_x, 2)
        a.x = a.x
        self.assertEqual(a._test_x, 3)
        a.x = None
        self.assertEqual(a._test_x, 4)

        a.y = numpy.arange(10)
        self.assertEqual(a._test_y, 1)
        a.y = 1.0
        self.assertEqual(a.y.shape, (1,))
        self.assertEqual(a._test_y, 2)
        a.y = a.y
        self.assertEqual(a._test_y, 3)
        a.y = None
        self.assertEqual(a._test_y, 4)


if __name__ == '__main__':
    unittest.main()




# Author: Suyog Dutt Jain <suyog.jain@aero.iitb.ac.in>
#         Prabhu Ramachandran <prabhu_r@users.sf.net>
# Copyright (c) 2008-2015,  Enthought, Inc.
# License: BSD Style.

# Standard library imports.
from os.path import abspath
from io import BytesIO
import copy
import numpy as np
import unittest

# Enthought library imports
from mayavi.core.null_engine import NullEngine
from mayavi.filters.contour import Contour
from mayavi.filters.optional import Optional
from mayavi.filters.collection import Collection
from mayavi.filters.api import PolyDataNormals
from mayavi.modules.api import Surface
from mayavi.sources.vtk_data_source import VTKDataSource

from mayavi.tests import datasets

class TestOptionalCollection(unittest.TestCase):

    def setUp(self):
        """Initial setting up of test fixture, automatically called by TestCase before any other test method is invoked"""
        e = NullEngine()
        # Uncomment to see visualization for debugging etc.
        #e = Engine()
        e.start()
        e.new_scene()
        self.e=e

        sgrid=datasets.generateStructuredGrid()
        src = VTKDataSource(data = sgrid)
        e.add_source(src)

        c = Contour()
        # `name` is used for the notebook tabs.
        n = PolyDataNormals(name='Normals')
        o = Optional(filter=n, label_text='Compute normals')
        coll = Collection(filters=[c, o], name='IsoSurface')
        e.add_filter(coll)
        s = Surface()
        e.add_module(s)
        self.coll = coll
        self.scene = e.current_scene
        return

    def tearDown(self):
        """For necessary clean up, automatically called by TestCase after the test methods have been invoked"""
        self.e.stop()
        return

    def check(self,coll):
        """Do the actual testing."""
        scene = self.scene
        """Check if test status is OK given the collection."""
        c, o = coll.filters
        c = c.filter
        n = o.filter

        r = coll.get_output_dataset().point_data.scalars.range

        self.assertEqual(np.allclose(r, (6.09,6.09), atol=1.01e-03), True)
        # Adding a contour should create the appropriate output in
        # the collection.
        c.contours.append(200)
        self.assertEqual(np.allclose(r, [6.09,6.09], atol=1.01e-03), True)
        # the collection's output should be that of the normals.
        self.assertEqual(coll.outputs[0] is n.outputs[0],True)
        # disable the optional filter and check.
        o.enabled = False
        self.assertEqual('disabled' in o.name,True)
        self.assertEqual(coll.outputs[0] is c.outputs[0],True)
        # Set back everything to original state.
        c.contours.pop()
        o.enabled = True
        self.assertEqual(np.allclose(r, (6.09,6.09), atol=1.01e-03), True)
        self.assertEqual(coll.outputs[0] is n.outputs[0], True)
        self.assertEqual('disabled' not in o.name, True)

    def test_optional_collection(self):
        "Test if the test fixture works"
        #Now test.
        coll = self.coll
        self.check(coll)

        #from mayavi.tools.show import show
        #show()

    def test_save_and_restore(self):
        """Test if saving a visualization and restoring it works."""
        engine = self.e
        scene = self.scene

        # Save visualization.
        f = BytesIO()
        f.name = abspath('test.mv2') # We simulate a file.
        engine.save_visualization(f)
        f.seek(0) # So we can read this saved data.

        # Remove existing scene.

        engine.close_scene(scene)

        # Load visualization
        engine.load_visualization(f)
        self.scene = engine.current_scene
        coll = self.scene.children[0].children[0]

        self.check(coll)


    def test_deepcopied(self):
        """Test if the MayaVi2 visualization can be deep-copied."""
        ############################################################
        # Test if the MayaVi2 visualization can be deep-copied.

        # Pop the source object.
        s =  self.scene
        source = s.children.pop()
        # Add it back to see if that works without error.
        s.children.append(source)
        coll = s.children[0].children[0]

        self.check(coll)

        # Now deepcopy the source and replace the existing one with
        # the copy.  This basically simulates cutting/copying the
        # object from the UI via the right-click menu on the tree
        # view, and pasting the copy back.
        source1 = copy.deepcopy(source)
        s.children[0] = source1
        coll = s.children[0].children[0]
        self.check(coll)
        #from mayavi.tools.show import show
        #show()



if __name__ == '__main__':
    unittest.main()



"""The `Wrapper` filter is one which wraps around any mayavi filter or
component.  By default it does not allow the user to set it on and off
from the UI, for that see the `Optional` filter.
"""

# Author: Prabhu Ramachandran <prabhu [at] aero . iitb . ac . in>
# Copyright (c) 2008,  Enthought, Inc.
# License: BSD Style.

# Enthought library imports.
from traits.api import Instance, Bool, Str
from traitsui.api import Item, Group, View
from apptools.persistence import state_pickler

# Local imports.
from mayavi.core.pipeline_base import PipelineBase
from mayavi.core.filter import Filter
from mayavi.core.common import handle_children_state

################################################################################
# `Wrapper` class.
################################################################################
class Wrapper(Filter):
    """
    The `Wrapper` filter is one which wraps around any mayavi filter or
    component.  By default it does not allow the user to set it on and
    off from the UI, for that see the `Optional` filter.
    """

    # The filter we wrap.
    filter = Instance(PipelineBase, allow_none=False, record=True)

    # The text to show in the UI of form "Enable SomeFilter"
    label_text = Str('Enable Filter')

    # Are we enabled or not.
    enabled = Bool(True, desc='if the filter is enabled or not')

    ########################################
    # Private traits.

    # Should we show enabled in the UI or not.  This defaults to False,
    # the `Optional` filter merely changes this to True.  This trait is
    # not meant for interactive changing.
    _show_enabled = Bool(False)

    ######################################################################
    # `object` interface.
    ######################################################################
    def __set_pure_state__(self, state):
        # Create and set the filter.
        children = [f for f in [self.filter] if f is not None]
        handle_children_state(children, [state.filter])
        self.filter = children[0]
        # Restore our state.
        super(Wrapper, self).__set_pure_state__(state)

    ######################################################################
    # HasTraits interface.
    ######################################################################
    def default_traits_view(self):
        """Returns the default traits view for this object."""
        if self._show_enabled:
            view = View(Group(Group(Item(name='enabled',
                                         label=self.label_text)),
                            Group(Item(name='filter',
                                       style='custom',
                                       enabled_when='enabled',
                                       resizable=True),
                                  show_labels=False)),
                        resizable=True)
        else:
            view = View(Group(Item(name='filter',
                                   style='custom',
                                   enabled_when='enabled',
                                   resizable=True),
                              show_labels=False),
                        resizable=True)

        return view

    ######################################################################
    # `Filter` interface.
    ######################################################################
    def setup_pipeline(self):
        """Setup the pipeline."""
        # Needed because a user may have defined a filter by setting the
        # default value of the trait in the subclass in which case the
        # filter changed handler will never be called leading to
        # problems.
        if self.filter is not None:
            self._setup_events(self.filter)

    def stop(self):
        # There is no need to override start since the wrapped filter is
        # always started automatically in the _enabled_changed handler.
        super(Wrapper, self).stop()
        if self.filter is not None:
            self.filter.stop()

    def update_pipeline(self):
        """This method *updates* the tvtk pipeline when data upstream is
        known to have changed.

        This method is invoked (automatically) when the input fires a
        `pipeline_changed` event.
        """
        self._enabled_changed(self.enabled)
        self.pipeline_changed = True

    def update_data(self):
        """This method does what is necessary when upstream data
        changes.

        This method is invoked (automatically) when any of the inputs
        sends a `data_changed` event.
        """
        # Propagate the data_changed event.
        self.data_changed = True

    ######################################################################
    # Private interface.
    ######################################################################
    def _enabled_changed(self, value):
        """Static traits handler."""
        if len(self.inputs) == 0 or self.filter is None:
            return
        my_input = self.inputs[0]
        filter = self.filter
        if len(filter.name) == 0:
            name = filter.__class__.__name__
        else:
            name = filter.name
        if value and filter is not None:
            filter.inputs = [my_input]
            if not filter.running:
                filter.start()
            self._set_outputs(self.filter.outputs)
        else:
            self._set_outputs(my_input.outputs)
            name += ' (disabled)'
        self.name = name
        self.render()

    def _filter_changed(self, old, new):
        """Static traits handler."""
        if old is not None:
            self._setup_events(old, remove=True)
            old.stop()
        if self.scene is not None:
            new.scene = self.scene
        self._setup_events(new, remove=False)
        self._enabled_changed(self.enabled)

    def _scene_changed(self, old, new):
        """Static traits handler."""
        if self.filter is not None:
            self.filter.scene = new
        super(Wrapper, self)._scene_changed(old, new)

    def _filter_pipeline_changed(self):
        if self.enabled:
            self._set_outputs(self.filter.outputs)

    def _setup_events(self, obj, remove=False):
        obj.on_trait_change(self._filter_pipeline_changed,
                            'pipeline_changed',
                            remove=remove)
        obj.on_trait_change(self.update_data,
                            'data_changed',
                            remove=remove)

    def _visible_changed(self, value):
        self.filter.visible = value
        super(Wrapper, self)._visible_changed(value)




{ 'array':

array([('Foo', 12.5, 85.4),
       ('Bar', 14.1, 87.6),
       ('Baz', 14.3, 89.0)],
      dtype=[('Sample Name', 'S3'), ('Radius', float), ('Speed', float)]),

  'kwds': { 'comments': '%',
            'delimiter': ':',
            'dtype': { 'formats': ('S3', float, float),
                       'names': ('Sample Name', 'Radius', 'Speed')},
            'skiprows': 3
            }
  }



# Author: Suyog Dutt Jain <suyog.jain@aero.iitb.ac.in>
# Copyright (c) 2009-2015,  Enthought, Inc.
# License: BSD Style.

# Standard library imports.
import unittest

# Local imports.
from mayavi.tests.common import get_example_data

# Enthought library imports
from mayavi.sources.poly_data_reader import PolyDataReader
from mayavi.tests.data_reader_test_base import DataReaderTestBase

# External library imports
import vtk

vtk_major_version = vtk.vtkVersion.GetVTKMajorVersion()
vtk_minor_version = vtk.vtkVersion.GetVTKMinorVersion()


class TestPDBReader(DataReaderTestBase):

    def setup_reader(self):

        """"Setup the reader in here.  This is called after the engine
        has been created and started.  The engine is available as
        self.e.  This method is called by setUp().
        """
        # Read a PDB data file.
        r = PolyDataReader()
        r.initialize(get_example_data('caffeine.pdb'))
        self.e.add_source(r)
        self.bounds = (3.10, 10.78, -2.39, 4.03, -10.60, -6.31)

    def test_pdb_data_reader(self):
        "Test if the test fixture works"

        self.check(self.scene, self.bounds)

    def test_save_and_restore(self):
        """Test if saving a visualization and restoring it works."""

        self.check_saving(self.e, self.scene, self.bounds)

    def test_deepcopied(self):
        """Test if the MayaVi2 visualization can be deep-copied."""
        ############################################################
        # Test if the MayaVi2 visualization can be deep-copied.

        self.check_deepcopying(self.scene, self.bounds)


class TestBYUReader(DataReaderTestBase):

    def setup_reader(self):

        """"Setup the reader in here.  This is called after the engine
        has been created and started.  The engine is available as
        self.e.  This method is called by setUp().
        """
        # Read a BYU data file.
        r = PolyDataReader()
        r.initialize(get_example_data('cow.g'))
        self.e.add_source(r)
        self.bounds = (-4.445, 5.998, -3.608, 2.760, -1.690, 1.690)

    def test_byu_data_reader(self):
        "Test if the test fixture works"

        self.check(self.scene, self.bounds)

    def test_save_and_restore(self):
        """Test if saving a visualization and restoring it works."""

        self.check_saving(self.e, self.scene, self.bounds)

    def test_deepcopied(self):
        """Test if the MayaVi2 visualization can be deep-copied."""
        ############################################################
        # Test if the MayaVi2 visualization can be deep-copied.

        self.check_deepcopying(self.scene, self.bounds)


class TestOBJReader(DataReaderTestBase):

    def setup_reader(self):

        """"Setup the reader in here.  This is called after the engine
        has been created and started.  The engine is available as
        self.e.  This method is called by setUp().
        """
        # Read a OBJ data file.
        r = PolyDataReader()
        r.initialize(get_example_data('shuttle.obj'))
        self.e.add_source(r)
        self.bounds = (-7.65, 7.04, -4.68, 4.68, -1.35, 4.16)

    def test_obj_data_reader(self):
        "Test if the test fixture works"

        self.check(self.scene, self.bounds)

    def test_save_and_restore(self):
        """Test if saving a visualization and restoring it works."""

        self.check_saving(self.e, self.scene, self.bounds)

    def test_deepcopied(self):
        """Test if the MayaVi2 visualization can be deep-copied."""
        ############################################################
        # Test if the MayaVi2 visualization can be deep-copied.

        self.check_deepcopying(self.scene, self.bounds)


class TestParticleReader(DataReaderTestBase):

    def setup_reader(self):

        """"Setup the reader in here.  This is called after the engine
        has been created and started.  The engine is available as
        self.e.  This method is called by setUp().
        """
        # Read a Particle data file.
        r = PolyDataReader()
        r.initialize(get_example_data('Particles.raw'))
        self.e.add_source(r)
        r.reader.trait_set(data_byte_order='big_endian', data_type='float',
                     file_type='binary')
        self.bounds = (817.33, 826.09, 545.02, 571.02, 1443.48, 1511.18)

    def test_particle_data_reader(self):
        "Test if the test fixture works"

        self.check(self.scene, self.bounds)

    def test_save_and_restore(self):
        """Test if saving a visualization and restoring it works."""

        self.check_saving(self.e, self.scene, self.bounds)

    def test_deepcopied(self):
        """Test if the MayaVi2 visualization can be deep-copied."""
        ############################################################
        # Test if the MayaVi2 visualization can be deep-copied.

        self.check_deepcopying(self.scene, self.bounds)


class TestPLYReader(DataReaderTestBase):

    def setup_reader(self):

        """"Setup the reader in here.  This is called after the engine
        has been created and started.  The engine is available as
        self.e.  This method is called by setUp().
        """
        # Read a PLY data file.
        r = PolyDataReader()
        r.initialize(get_example_data('pyramid.ply'))
        self.e.add_source(r)
        self.bounds = (0.0, 1.0, 0.0, 1.0, 0.0, 1.60)

    def test_ply_data_reader(self):
        self.check(self.scene, self.bounds)

    def test_save_and_restore(self):
        self.check_saving(self.e, self.scene, self.bounds)

    def test_deepcopied(self):
        """Test if the MayaVi2 visualization can be deep-copied."""
        self.check_deepcopying(self.scene, self.bounds)


class TestPointsReader(DataReaderTestBase):

    def setup_reader(self):

        """"Setup the reader in here.  This is called after the engine
        has been created and started.  The engine is available as
        self.e.  This method is called by setUp().
        """
        # Read a Points data file.
        r = PolyDataReader()
        r.initialize(get_example_data('points.txt'))
        self.e.add_source(r)
        self.bounds = (0.0, 1.0, 0.0, 1.0, 0.0, 1.0)

    def test_points_data_reader(self):
        "Test if the test fixture works"

        self.check(self.scene, self.bounds)

    def test_save_and_restore(self):
        """Test if saving a visualization and restoring it works."""

        self.check_saving(self.e, self.scene, self.bounds)

    def test_deepcopied(self):
        """Test if the MayaVi2 visualization can be deep-copied."""
        ############################################################
        # Test if the MayaVi2 visualization can be deep-copied.

        self.check_deepcopying(self.scene, self.bounds)


class TestSTLReader(DataReaderTestBase):

    def setup_reader(self):

        """"Setup the reader in here.  This is called after the engine
        has been created and started.  The engine is available as
        self.e.  This method is called by setUp().
        """
        # Read a STL data file.
        r = PolyDataReader()
        r.initialize(get_example_data('humanoid_tri.stla'))
        self.e.add_source(r)
        self.bounds = (0.60, 3.47, -3.96, 3.95, 3.05, 17.39)

    def test_stl_data_reader(self):
        "Test if the test fixture works"

        self.check(self.scene, self.bounds)

    def test_save_and_restore(self):
        """Test if saving a visualization and restoring it works."""

        self.check_saving(self.e, self.scene, self.bounds)

    def test_deepcopied(self):
        """Test if the MayaVi2 visualization can be deep-copied."""
        ############################################################
        # Test if the MayaVi2 visualization can be deep-copied.

        self.check_deepcopying(self.scene, self.bounds)


class TestFacetReader(DataReaderTestBase):

    def setup_reader(self):

        """"Setup the reader in here.  This is called after the engine
        has been created and started.  The engine is available as
        self.e.  This method is called by setUp().
        """
        # Read a Facet data file.
        r = PolyDataReader()
        r.initialize(get_example_data('clown.facet'))
        self.e.add_source(r)
        self.bounds = (-0.5, 0.69, -0.49, 0.49, -1.09, 0.5)

    def test_facet_data_reader(self):
        "Test if the test fixture works"

        self.check(self.scene, self.bounds)

    def test_save_and_restore(self):
        """Test if saving a visualization and restoring it works."""

        self.check_saving(self.e, self.scene, self.bounds)

    def test_deepcopied(self):
        """Test if the MayaVi2 visualization can be deep-copied."""
        ############################################################
        # Test if the MayaVi2 visualization can be deep-copied.

        self.check_deepcopying(self.scene, self.bounds)


class TestSLCReader(DataReaderTestBase):

    def setup_reader(self):

        """"Setup the reader in here.  This is called after the engine
        has been created and started.  The engine is available as
        self.e.  This method is called by setUp().
        """
        # Read a SLC data file.
        r = PolyDataReader()
        r.initialize(get_example_data('nut.slc'))
        self.e.add_source(r)
        self.bounds = (0.0, 67.0, 0.0, 40.0, 0.0, 58.0)

    def test_slc_data_reader(self):
        "Test if the test fixture works"

        self.check(self.scene, self.bounds)

    def test_save_and_restore(self):
        """Test if saving a visualization and restoring it works."""

        self.check_saving(self.e, self.scene, self.bounds)

    def test_deepcopied(self):
        """Test if the MayaVi2 visualization can be deep-copied."""
        ############################################################
        # Test if the MayaVi2 visualization can be deep-copied.

        self.check_deepcopying(self.scene, self.bounds)

if __name__ == '__main__':
    unittest.main()
